package mqtt_api

import (
	"errors"
	"fmt"
	"git.sxidc.com/go-tools/utils/strutils"
	mqtt "github.com/eclipse/paho.mqtt.golang"
	"time"
)

type MqttClientOptions struct {
	UserName        string
	Password        string
	Address         string
	ClientID        string
	KeepAliveSec    int64
	PingTimeoutSec  int64
	WriteTimeoutSec int64
}

func (opts *MqttClientOptions) check() error {
	if strutils.IsStringEmpty(opts.UserName) {
		return errors.New("必须传递用户名")
	}

	if strutils.IsStringEmpty(opts.Password) {
		return errors.New("必须传递密码")
	}

	if strutils.IsStringEmpty(opts.Address) {
		return errors.New("必须传递地址")
	}

	if strutils.IsStringEmpty(opts.ClientID) {
		return errors.New("必须传递客户端ID")
	}

	return nil
}

type MqttClient struct {
	mqttClient mqtt.Client
	opts       *MqttClientOptions
}

func newMqttClient(opts *MqttClientOptions, onConnect func(client *MqttClient), onConnectionLost func(client *MqttClient)) (*MqttClient, error) {
	if opts == nil {
		return nil, errors.New("没有传递mqtt客户端选项")
	}

	err := opts.check()
	if err != nil {
		return nil, err
	}

	if opts.WriteTimeoutSec == 0 {
		opts.WriteTimeoutSec = 60
	}

	mqttClient := &MqttClient{
		opts: opts,
	}

	mqttOptions := mqtt.NewClientOptions().
		SetAutoReconnect(true).
		SetUsername(opts.UserName).
		SetPassword(opts.Password).
		AddBroker(opts.Address).
		SetClientID(opts.ClientID).
		SetKeepAlive(time.Duration(opts.KeepAliveSec)*time.Second).
		SetPingTimeout(time.Duration(opts.PingTimeoutSec)*time.Second).
		SetWriteTimeout(time.Duration(opts.WriteTimeoutSec)*time.Second).
		SetWill(opts.ClientID+"/will", "dead", 2, false).
		SetOnConnectHandler(func(client mqtt.Client) {
			onConnect(mqttClient)
		}).
		SetConnectionLostHandler(func(client mqtt.Client, _ error) {
			onConnectionLost(mqttClient)
		})

	mqttClient.mqttClient = mqtt.NewClient(mqttOptions)

	return mqttClient, nil
}

func destroyMqttClient(c *MqttClient) {
	if c != nil {
		c.mqttClient = nil
	}

	c = nil
}

func (client *MqttClient) connect() error {
	token := client.mqttClient.Connect()
	if !token.WaitTimeout(time.Duration(client.opts.WriteTimeoutSec) * time.Second) {
		return errors.New("连接超时")
	}

	if token.Error() != nil {
		return token.Error()
	}

	return nil
}

func (client *MqttClient) disconnect() {
	client.mqttClient.Disconnect(250)
}

func (client *MqttClient) subscribe(topic string, callback func(topic string, data []byte)) error {
	token := client.mqttClient.Subscribe(topic, 2, func(mqttClient mqtt.Client, message mqtt.Message) {
		callback(topic, message.Payload())
	})
	if !token.WaitTimeout(time.Duration(client.opts.WriteTimeoutSec) * time.Second) {
		return errors.New("订阅超时")
	}

	if token.Error() != nil {
		return token.Error()
	}

	fmt.Println("[MQTT] Subscribe Topic: " + topic)

	return nil
}

func (client *MqttClient) unsubscribe(topic string) error {
	token := client.mqttClient.Unsubscribe(topic)
	if !token.WaitTimeout(time.Duration(client.opts.WriteTimeoutSec) * time.Second) {
		return errors.New("取消订阅超时")
	}

	if token.Error() != nil {
		return token.Error()
	}

	fmt.Println("[MQTT] Unsubscribe Topic: " + topic)

	return nil
}

func (client *MqttClient) response(topic string, responseIdentifier string, data []byte) error {
	replyTopic := topic + "/reply"
	
	if strutils.IsStringNotEmpty(responseIdentifier) {
		replyTopic = fmt.Sprintf("%s/%s/reply", topic, responseIdentifier)
	}

	token := client.mqttClient.Publish(replyTopic, 2, false, data)
	if !token.WaitTimeout(time.Duration(client.opts.WriteTimeoutSec) * time.Second) {
		return errors.New("发布超时")
	}

	if token.Error() != nil {
		return token.Error()
	}

	return nil
}
