package mqtt_response

import (
	"encoding/json"
	"fmt"
	"git.sxidc.com/go-framework/baize/framework/core/infrastructure/logger"
	"git.sxidc.com/go-framework/baize/framework/core/mqtt_api"
)

type SendResponseFunc[T any] func(c *mqtt_api.Context, identifier string, data T, err error)

func NoResponse(_ *mqtt_api.Context, _ string, _ any, _ error) {
	return
}

func SendMsgResponse(c *mqtt_api.Context, identifier string, _ any, err error) {
	msgResp := formMsgResponse(err)
	jsonResponse(c, identifier, msgResp)
}

type IDResponse struct {
	MsgResponse
	ID string `json:"id"`
}

func SendIDResponse(c *mqtt_api.Context, identifier string, id string, err error) {
	msgResp := formMsgResponse(err)
	jsonResponse(c, identifier, IDResponse{
		MsgResponse: msgResp,
		ID:          id,
	})
}

type InfoResponse[T any] struct {
	MsgResponse
	Info T `json:"info"`
}

func SendInfoResponse[T any](c *mqtt_api.Context, identifier string, info T, err error) {
	msgResp := formMsgResponse(err)
	jsonResponse(c, identifier, InfoResponse[T]{
		MsgResponse: msgResp,
		Info:        info,
	})
}

type InfosData[T any] struct {
	Infos      []T   `json:"infos"`
	TotalCount int64 `json:"totalCount"`
	PageNo     int   `json:"pageNo"`
}

type InfosResponse[T any] struct {
	MsgResponse
	InfosData[T]
}

func SendInfosResponse[T any](c *mqtt_api.Context, identifier string, data InfosData[T], err error) {
	msgResp := formMsgResponse(err)
	jsonResponse(c, identifier, InfosResponse[T]{
		MsgResponse: msgResp,
		InfosData:   data,
	})
}

func StructToMap(originStruct any) map[string]any {
	jsonBytes, err := json.Marshal(originStruct)
	if err != nil {
		panic(err)
	}

	retMap := make(map[string]any)
	err = json.Unmarshal(jsonBytes, &retMap)
	if err != nil {
		panic(err)
	}

	return retMap
}

func SendMapResponse(c *mqtt_api.Context, identifier string, data map[string]any, err error) {
	msgRespMap := formMapMsgResponse(err)
	for key, value := range data {
		msgRespMap[key] = value
	}

	jsonResponse(c, identifier, msgRespMap)
}

type MsgResponse struct {
	Success bool   `json:"success"`
	ErrCode int    `json:"errCode"`
	Msg     string `json:"msg"`
}

func formMsgResponse(err error) MsgResponse {
	if err != nil {
		logger.GetInstance().Error(err)

		return MsgResponse{
			Success: false,
			ErrCode: 1,
			Msg:     err.Error(),
		}
	}

	return MsgResponse{
		Success: true,
		ErrCode: 0,
		Msg:     "操作成功",
	}
}

func formMapMsgResponse(err error) map[string]any {
	resp := make(map[string]any)
	if err != nil {
		logger.GetInstance().Error(err)

		resp["success"] = false
		resp["errCode"] = 1
		resp["msg"] = err.Error()

		return resp
	}

	resp["success"] = true
	resp["errCode"] = 0
	resp["msg"] = "操作成功"

	return resp
}

func SendString(c *mqtt_api.Context, identifier string, data string, err error) {
	if err != nil {
		bytesResponse(c, identifier, []byte(err.Error()))
		return
	}

	bytesResponse(c, identifier, []byte(data))
}

func WriteBytes(c *mqtt_api.Context, identifier string, bytes []byte, err error) {
	if err != nil {
		bytesResponse(c, identifier, []byte(err.Error()))
		return
	}

	bytesResponse(c, identifier, bytes)
}

func jsonResponse(c *mqtt_api.Context, identifier string, obj any) {
	jsonBytes, err := json.Marshal(obj)
	if err != nil {
		panic(err)
	}

	err = c.Response(identifier, jsonBytes)
	if err != nil {
		fmt.Println("发送mqtt json响应失败: " + err.Error())
		return
	}
}

func bytesResponse(c *mqtt_api.Context, identifier string, obj []byte) {
	err := c.Response(identifier, obj)
	if err != nil {
		fmt.Println("发送mqtt bytes响应失败")
		return
	}
}
